@@ -1,3 +1,5 @@
+mod error;
+
 use core::fmt::Debug;
 use core::mem;
 use core::task::Waker;
@@ -5,8 +7,12 @@ use core::task::Waker;
 use parking_lot::const_mutex;
 use parking_lot::Mutex;
 
+pub use error::Error;
+
 #[derive(Debug)]
 enum InnerState<Input, Output> {
+    Uninitialized,
+
     Ongoing(Option<Input>, Option<Waker>),
 
     /// The awaitable is done
@@ -14,18 +20,16 @@ enum InnerState<Input, Output> {
 
     Consumed,
 }
-impl<Input, Output> InnerState<Input, Output> {
-    const fn new(input: Option<Input>) -> Self {
-        InnerState::Ongoing(input, None)
-    }
-}
 
 #[derive(Debug)]
 pub struct Awaitable<Input, Output>(Mutex<InnerState<Input, Output>>);
 
 impl<Input, Output> Awaitable<Input, Output> {
-    pub const fn new(input: Option<Input>) -> Self {
-        Self(const_mutex(InnerState::new(input)))
+    /// Create an uninitialized `Awaitable`.
+    ///
+    /// Must be `reset` before it can be used.
+    pub const fn new() -> Self {
+        Self(const_mutex(InnerState::Uninitialized))
     }
 }
 
@@ -45,60 +49,63 @@ impl<Input: Debug, Output: Debug> Awaitable<Input, Output> {
     /// `install_waker` must not be registered twice.
     /// `install_waker` must not be called after `take_output` is called.
     /// **
-    pub fn install_waker(&self, waker: Waker) -> bool {
+    pub fn install_waker(&self, waker: Waker) -> Result<bool, Error> {
         use InnerState::*;
 
         let mut guard = self.0.lock();
 
         match &mut *guard {
+            Uninitialized => Err(Error::Uninitialized),
+
             Ongoing(_input, stored_waker) => {
                 if stored_waker.is_some() {
-                    panic!("Waker is installed twice before the awaitable is done");
+                    Err(Error::WakerAlreadyInstalled)
+                } else {
+                    *stored_waker = Some(waker);
+                    Ok(false)
                 }
-                *stored_waker = Some(waker);
-                false
-            }
-            Done(_) => true,
-            Consumed => {
-                panic!("Waker is installed after the awaitable is done and its result consumed")
             }
+            Done(_) => Ok(true),
+            Consumed => Err(Error::AlreadyConsumed),
         }
     }
 
     /// **`take_input` must not be called after `take_output` is called.
-    pub fn take_input(&self) -> Option<Input> {
+    pub fn take_input(&self) -> Result<Option<Input>, Error> {
         use InnerState::*;
 
         let mut guard = self.0.lock();
 
         match &mut *guard {
-            Ongoing(input, _stored_waker) => input.take(),
-            Done(_) => None,
-            Consumed => {
-                panic!("Task attempts to retrieve input after the awaitable is done and its result consumed")
-            }
+            Uninitialized => Err(Error::Uninitialized),
+
+            Ongoing(input, _stored_waker) => Ok(input.take()),
+            Done(_) => Ok(None),
+            Consumed => Err(Error::AlreadyConsumed),
         }
     }
 
     /// `done` must be only called once on one instance of `Awaitable`.
     ///
     /// **`done` must not be called after `take_output` is called.**
-    pub fn done(&self, value: Output) {
+    pub fn done(&self, value: Output) -> Result<(), Error> {
         use InnerState::*;
 
         let prev_state = mem::replace(&mut *self.0.lock(), Done(value));
 
         match prev_state {
-            Done(_) => panic!("Awaitable is marked as done twice"),
+            Uninitialized => Err(Error::Uninitialized),
+
+            Done(_) => Err(Error::AlreadyDone),
             Ongoing(_input, stored_waker) => {
                 if let Some(waker) = stored_waker {
                     waker.wake();
                 }
+
+                Ok(())
             }
-            Consumed => {
-                panic!("Awaitable is marked as done again after its result consumed")
-            }
-        };
+            Consumed => Err(Error::AlreadyConsumed),
+        }
     }
 
     /// Return `Some(output)` if the awaitable is done.
@@ -112,4 +119,9 @@ impl<Input: Debug, Output: Debug> Awaitable<Input, Output> {
             _ => None,
         }
     }
+
+    /// Return true if current state is `Done`.
+    pub fn is_done(&self) -> bool {
+        matches!(&*self.0.lock(), InnerState::Done(_))
+    }
 }
